/**
 * correlator_plugin.h
 *
 * Apama Plugin C API definitions.  Used by plugin libraries and the plugin
 * support code within the Engine itself.
 *
 * $Copyright(c) 2002, 2004-2006, 2008-2009, 2011-2013 Progress Software Corporation (PSC). All rights reserved.$
 * $Copyright (c) 2013, 2016-2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 * $Id: correlator_plugin.h 339905 2018-10-25 12:25:38Z antm $
 */

/** @file
 * @deprecated 
		This API is deprecated in favour of the C++ API defined in epl_plugin.hpp
 *
 * Contains the C API for writing EPL plugins.
 */

#ifndef CORRELATOR_PLUGIN_H
#define CORRELATOR_PLUGIN_H

#ifndef  _AP_NO_DEPRECATED_PLUGINAPI_WARNING
#ifdef __GNUC__
#warning This API for writing EPL plugins is deprecated. Please use the API located in epl_plugin.hpp instead
#else
#pragma message("correlator_plugin.h(0) Warning: This API for writing EPL plugins is deprecated. Please use the API located in epl_plugin.hpp instead")
#endif
#endif

#include <AP_Types.h>
#include <AP_Platform.h>
#include <AP_PluginCommon.h>
#include <stdio.h>


/** @deprecated
 * Plugin API version.  The upper 16 bits identify the major release of the
 * API; the lower 16 bits indicate minor versions within that release.  No
 * changes that break backward binary compatibility will be made within a
 * given major release of the API.  That is, minor releases may add
 * functionality and fix bugs, but will not change or remove any documented
 * behaviour.  AP_PluginCommon.h provides two _VERSION_MASK symbols for
 * convenience in performing fine-grained version checks.
 */
/* Version history:
 * 1.0	First version released in Apama Engine 2.0.3
 *	Win32 support added in Apama Engine 2.0.4
 * 2.0	Add serialisation support for Apama Engine 2.1 release
 *      Unfortunately this breaks back compatibility of chunks
 * 3.0  First C-compatible version of the API, in Apama Engine 2.1.2
 *      Note that the 2.0 C++ API still exists in this release; the
 *      2.x (C++) and 3.x (C) versions will be maintained in parallel.
 * 4.0  Major version number change to force users to re-compile for VS7.1
 *      and/or check the correct free implementation is used (result of
 *      InputStream's getString must be freed with AP_Context's char8free)
 *      Also adds support for the getCorrelator() context function and the
 *      AP_PluginCorrelatorInterface type that can be used to send events
 *      back into the correlator.  This brings the C++ and C plugin APIs
 *      back into sync - future changes should be made to both APIs.
 * 5.0  Add getContextId and sendEventTo. Also added the optional
 *      GET_CAPABILITIES and THREAD_ENDED functions that the plugin may
 *      provide. Improved some function prototypes.
 * 6.0  Improved the interface's const-correctness. Increased the
 *      performance of AP_TypeList, made AP_Chunk construction and
 *      destruction cheaper, especially in a multi-threaded
 *      environment. Added AP_Type::visitSequenceElements.
 * 7.0  Remove serialisation support for Apama 5.0 release
 * 8.0  No changes here, but to keep it in sync with the .hpp
 *	9.0  Add support for modifying sequence lengths and returning sequences
 */
#define AP_CORRELATOR_PLUGIN_VERSION      0x00090000

/** @deprecated 
		Typed differentiator for different exceptions in the plugin interface. */
typedef enum {
	/** Generic exception */
	AP_PLUGIN_EXCEPTION, 
	/** Incorrect type */
	AP_PLUGIN_TYPE_EXCEPTION, 
	/** Unimplemented method */
	AP_PLUGIN_UNIMPLEMENTED_EXCEPTION,
	/** Out of bands */
	AP_PLUGIN_BOUNDS_EXCEPTION, 
	AP_PLUGIN_SERIALISATION_EXCEPTION
} AP_CorrelatorPluginExceptionType;



#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

	struct AP_PluginCorrelatorInterface_API;
	struct AP_PluginContext_API;
	struct AP_PluginType_API;
	struct AP_PluginChunk_API;
	struct AP_PluginTypeList_API;

	struct AP_PluginChunk_Callbacks;

#ifdef __cplusplus
	struct AP_PluginCorrelatorInterface_Stubs;
	struct AP_PluginContext_Stubs;
	struct AP_PluginType_Stubs;
	struct AP_PluginChunk_Stubs;
	struct AP_PluginTypeList_Stubs;

	typedef struct AP_PluginCorrelatorInterface_Stubs AP_PluginCorrelatorInterface;
	typedef struct AP_PluginContext_Stubs AP_PluginContext;
	typedef struct AP_PluginType_Stubs AP_PluginType;
	typedef struct AP_PluginChunk_Stubs AP_PluginChunk;
	typedef struct AP_PluginTypeList_Stubs AP_PluginTypeList;
#else
	typedef struct AP_PluginCorrelatorInterface_API AP_PluginCorrelatorInterface;
	typedef struct AP_PluginContext_API AP_PluginContext;
	typedef struct AP_PluginType_API AP_PluginType;
	typedef struct AP_PluginChunk_API AP_PluginChunk;
	typedef struct AP_PluginTypeList_API AP_PluginTypeList;
#endif /* __cplusplus */

	/** [DEPRECATED]\n
	 * CorrelatorInterface function API.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginCorrelatorInterface_Functions
	{
		/** @deprecated
		 * Send an event to the correlator
		 *
		 * The event parameter is the event to send.  The event is represented as a string
		 * using the format described in the manual.
		 * See AP_EventWriter.h for utility code to help build valid event strings.
		 *
		 * @param sendEvent: This function. 
		 */
		void (AP_PLUGIN_CALL* sendEvent)(AP_PluginCorrelatorInterface* _interface, const char* event);
		/** @deprecated
		 * Send an event to a specific context in the correlator
		 *
		 * The event paramter is the event to send.  The event is represented as a string
		 * using the format described in the manual.
		 * See AP_EventWriter.h for utility code to help build valid event strings.
		 *
		 * The targetContext parameter is the target context.
		 *
		 * The sourceContext parameter is the source context. Must be as returned from
		 * AP_Context.getContextId(), or 0 if called from a plugin thread.
		 *
		 * @param sendEventTo: This function. 
		 */
		void (AP_PLUGIN_CALL* sendEventTo)(AP_PluginCorrelatorInterface* _interface, const char* event, AP_uint64 targetContext, AP_uint64 sourceContext);
	};

	/** [DEPRECATED]\n
	 * CorrelatorInterface.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginCorrelatorInterface_API
	{
		void* reserved;
		/** @deprecated 
		Pointer to the function API */
		const struct AP_PluginCorrelatorInterface_Functions* functions;
	};

	struct AP_PluginCorrelatorInterface_Stubs {
		struct AP_PluginCorrelatorInterface_API api;

#ifdef __cplusplus
		void sendEvent(char* event) {
			api.functions->sendEvent(this, event);
		}

		void sendEvent(char* event, AP_uint64 targetCtx, AP_uint64 sourceCtx) {
			api.functions->sendEventTo(this, event, targetCtx, sourceCtx);
		}
#endif /* __ cplusplus */
	};

	/** [DEPRECATED]\n
	 * PluginContext function API.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginContext_Functions
	{
		/** @deprecated 
		Return active plugin API version
	  @param version: this function. 
		 */
		AP_uint32 (AP_PLUGIN_CALL* version)(const AP_PluginContext* ctx);
		/** @deprecated
		 * Create a chunk. userData will always be freed using the supplied callback after using this
		 * in a call. You should not explicitly delete this chunk or the supplied userData.
		 * @param ctx: the plug-in context.
		 * @param userData a void pointer to your data to be held in the chunk
		 * @param callbacks a function pointer structure to free or copy userData
		 */
		AP_PluginChunk* (AP_PLUGIN_CALL* createChunk)(const AP_PluginContext* ctx, const struct AP_PluginChunk_Callbacks* callbacks, void* userData);
		/** @deprecated
		 * @internal
		 * Delete a chunk. This probably needs to be removed. It's not safe to call it once you've given
		 * a chunk to the correlator.
		 * @param ctx: the plug-in context.
		 * @param chunk: the chunk to be deleted.
		 */
		void (AP_PLUGIN_CALL* deleteChunk)(const AP_PluginContext* ctx, AP_PluginChunk* chunk);
		/** @deprecated 
		Returns true if an exception has occurred since the last time that clearException was called */
		AP_bool (AP_PLUGIN_CALL* checkException)(const AP_PluginContext* ctx);
		/** @deprecated 
		Returns the type of the most recent exception */
		AP_CorrelatorPluginExceptionType (AP_PLUGIN_CALL* getExceptionType)(const AP_PluginContext* ctx);
		/** @deprecated 
		Returns the message of the most recent exception */
		const AP_char8* (AP_PLUGIN_CALL* getExceptionMessage)(const AP_PluginContext* ctx);
		/** @deprecated 
		Clears the flag of whether an exception has occurred */
		void (AP_PLUGIN_CALL* clearException)(const AP_PluginContext* ctx);
		/** @deprecated
		 * Return the correlator implementation.
		 */
		AP_PluginCorrelatorInterface* (AP_PLUGIN_CALL* getCorrelator)(const AP_PluginContext* ctx);
		/** @deprecated
		 * Free the memory occupied by a string created by the plugin API.
		 * ptr is the source string to be freed.
		 * @param char8free This function. 
		 */
		void (AP_PLUGIN_CALL* char8free)(AP_char8* ptr);
		/** @deprecated
		 * Return the identifier of the EPL (parallelism) context associated
		 * with this plugin execution context.
		 */
		AP_uint64 (AP_PLUGIN_CALL* getContextId)(const AP_PluginContext* ctx);
	};

	/** [DEPRECATED]\n
	 * PluginContext.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginContext_API
	{
		const void* reserved;
		/** @deprecated 
		Pointer to the function API */
		const struct AP_PluginContext_Functions* functions;
	};

	struct AP_PluginContext_Stubs {
		struct AP_PluginContext_API api;

#ifdef __cplusplus
		AP_uint32 version() const {
			return api.functions->version(this);
		}

		AP_PluginChunk* createChunk(const struct AP_PluginChunk_Callbacks* callbacks, void* userData) const {
			return api.functions->createChunk(this, callbacks, userData);
		}

		void deleteChunk(AP_PluginChunk* chunk) const {
			api.functions->deleteChunk(this, chunk);
		}

		AP_bool checkException() const {
			return api.functions->checkException(this);
		}

		AP_CorrelatorPluginExceptionType getExceptionType() const {
			return api.functions->getExceptionType(this);
		}

		const AP_char8* getExceptionMessage() const {
			return api.functions->getExceptionMessage(this);
		}

		void clearException() const {
			api.functions->clearException(this);
		}
		
		const AP_PluginCorrelatorInterface* getCorrelator() const{
			return api.functions->getCorrelator(this);
		}

		void char8free(AP_char8* ptr) const{
			api.functions->char8free(ptr);
		}

		AP_uint64 getContextId() const{
			return api.functions->getContextId(this);
		}
#endif /* __ cplusplus */
	};


	/** [DEPRECATED]\n
	* PluginChunk function API.
	*
	* @deprecated This class has been deprecated.
	*
	*/
	struct AP_PluginChunk_Functions 
	{
		/** @deprecated 
		Return the user data stored in this chunk */
		void* (AP_PLUGIN_CALL* getUserData)(const AP_PluginChunk* chunk);
		/** @deprecated 
		
		 * @internal
		 * Replace the user data stored in this chunk.
		 * This does not delete the old userData, so the user must take care of that
		 */
		void (AP_PLUGIN_CALL* setUserData)(AP_PluginChunk* chunk, void* userData);
	};

	/** [DEPRECATED]\n
	 * PluginChunk callback API. You must implement one of these to 
	 * copy and free your user data.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginChunk_Callbacks
	{
		/** @deprecated 
		Free the specified user data */
		void (AP_PLUGIN_CALL* freeUserData)(AP_PluginChunk* chunk);
		/** @deprecated 
		return a copy of the user data */
		void* (AP_PLUGIN_CALL* copyUserData)(const AP_PluginChunk* chunk);
	};

	/** [DEPRECATED]\n
	 * PluginChunk.
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginChunk_API
	{
		void* reserved;
		/** @deprecated 
		Pointer to function API */
		const struct AP_PluginChunk_Functions *functions;
		/** @deprecated 
		Pointer to the user callbacks for this chunk */
		const struct AP_PluginChunk_Callbacks *callbacks;
	};

	struct AP_PluginChunk_Stubs {
		struct AP_PluginChunk_API api;

#ifdef __cplusplus
		void* getUserData() const {
			return api.functions->getUserData(this);
		}

		void setUserData(void* userData) {
			api.functions->setUserData(this, userData);
		}

		void freeUserData() {
			api.callbacks->freeUserData(this);
		}

		void* copyUserData() {
			return api.callbacks->copyUserData(this);
		}
#endif /* __cplusplus */
	};



	/** [DEPRECATED]\n
	* AP_Type function API
	* 
	* @deprecated This class has been deprecated.
	* 
	*/
	struct AP_PluginType_Functions
	{
		/** @deprecated 
		Return the type contained in this object */
		AP_TypeDiscriminator (AP_PLUGIN_CALL* getDiscriminator)(const AP_PluginType* obj);

		/** @deprecated 
		Get the chunk value of the object */
		AP_PluginChunk* (AP_PLUGIN_CALL* getChunkValue)(const AP_PluginType* obj);
		/** @deprecated
		 * Set the chunk value of the object.
		 *
		 * The correlator takes ownership of the new AP_PluginChunk; it should
		 * not be deleted by the plugin.
		 *
		 * The new pointer may be null.
		 */
		void (AP_PLUGIN_CALL* setChunkValue)(const AP_PluginType* obj, AP_PluginChunk* val);

		/** @deprecated 
		Get the integer value of the object */
		AP_int64 (AP_PLUGIN_CALL* getIntegerValue)(const AP_PluginType* obj);
		/** @deprecated 
		Set the integer value of the object */
		void (AP_PLUGIN_CALL* setIntegerValue)(AP_PluginType* obj, AP_int64 val);

		/** @deprecated 
		Get the float value of the object */
		AP_float64 (AP_PLUGIN_CALL* getFloatValue)(const AP_PluginType* obj);
		/** @deprecated 
		Set the float value of the object */
		void (AP_PLUGIN_CALL* setFloatValue)(AP_PluginType* obj, AP_float64 val);

		/** @deprecated 
		Get the decimal value of the object */
		AP_decimal (AP_PLUGIN_CALL* getDecimalValue)(const AP_PluginType* obj);
		/** @deprecated 
		Set the decimal value of the object */
		void (AP_PLUGIN_CALL* setDecimalValue)(AP_PluginType* obj, AP_decimal val);

		/** @deprecated 
		Get the boolean value of the object */
		AP_bool (AP_PLUGIN_CALL* getBooleanValue)(const AP_PluginType* obj);
		/** @deprecated 
		Set the boolean value of the object */
		void (AP_PLUGIN_CALL* setBooleanValue)(AP_PluginType* obj, AP_bool val);

		/** @deprecated 
		Get the string value of the object */
		const AP_char8* (AP_PLUGIN_CALL* getStringValue)(const AP_PluginType* obj);
		/** @deprecated 
		Set the string value of the object.
		 * Note that this is only possible on sequence elements and
		 * return values, not arguments (which are const)
		 */
		void (AP_PLUGIN_CALL* setStringValue)(AP_PluginType* obj, const AP_char8* val);

		/** @deprecated 
		Get the number of elements in a sequence object */
		AP_uint32 (AP_PLUGIN_CALL* getSequenceLength)(const AP_PluginType* obj);
		/** @deprecated 
		Set the length of a sequence object.
		 * Increasing the length will populate the sequence with default-initialised objects.
		 * Decreasing the length will remove items from the end.
		 *
		 * You are not permitted to call setSequenceLength after calling sequenceElements without releasing the elements.
		 * If you call setSequenceLength any references returned from sequenceElement or operator[] are invalid and may not be used.
		 */
		void (AP_PLUGIN_CALL* setSequenceLength)(const AP_PluginType* obj, AP_uint32 len);
		/** @deprecated 
		Get the element type of a sequence object */
		AP_TypeDiscriminator (AP_PLUGIN_CALL* getSequenceType)(const AP_PluginType* obj);
		/** @deprecated
		 * Take an uninitialised sequence object and create an empty sequence with
		 * the indicated member type.
		 *
		 * Member type can only be primitives, string or chunk.
		 *
		 * Typically used for return values from functions.
		 */
		void (AP_PLUGIN_CALL* createSequence)(const AP_PluginType *obj, AP_TypeDiscriminator inner);

		/** @deprecated
		 * Get a pointer to a single sequence element. The
		 * element AP_Type remains usable until the sequence AP_Type
		 * is destroyed.
		 */
		AP_PluginType* (AP_PLUGIN_CALL* getSequenceElement)(const AP_PluginType* obj, AP_uint32 index);

		/** @deprecated 
		Copy another AP_Type into this, regardless of type */
		void (AP_PLUGIN_CALL *copyFrom)(const AP_PluginType *obj, const AP_PluginType *other);

		/** @deprecated
		 * Generate an array of pointers to AP_Type objects, encapsulating
		 * elements [start..start+length-1] of the sequence object.  This may
		 * or may not require copying the original EPL objects.
		 */
		AP_PluginType* const * (AP_PLUGIN_CALL* getSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * sequenceElements() and ensures that any changes made to the
		 * sequence elements are reflected in the underlying EPL
		 * objects.  Any arrays previously returned by sequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of pointers to chunks encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects.
		 *
		 * Note that the AP_Chunk pointers cannot be modified via this
		 * call. Use sequenceElement(), visitSequenceElements() or
		 * sequenceElements() instead, being sure to free any AP_Chunk
		 * instances that are replaced.
		 */
		AP_PluginChunk* const * (AP_PLUGIN_CALL* getChunkSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * chunkSequenceElements(). Arrays previously returned by
		 * chunkSequenceElements() immediately become invalid. This
		 * function is automatically called when a sequence AP_Type is
		 * destroyed, but can be invoked explicitly by a plugin during
		 * a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseChunkSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of integers encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects.
		 */
		AP_int64* (AP_PLUGIN_CALL* getIntegerSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * integerSequenceElements() and ensures that any changes made to the
		 * sequence elements are reflected in the underlying EPL
		 * objects.  Arrays previously returned by integerSequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseIntegerSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of floats encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects.
		 */
		AP_float64* (AP_PLUGIN_CALL* getFloatSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * floatSequenceElements() and ensures that any changes made to the
		 * sequence elements are reflected in the underlying EPL
		 * objects.  Arrays previously returned by floatSequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseFloatSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of decimals encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects.
		 */
		AP_decimal* (AP_PLUGIN_CALL* getDecimalSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * getDecimalSequenceElements() and ensures that any changes made to the
		 * sequence elements are reflected in the underlying EPL
		 * objects.  Arrays previously returned by getDecimalSequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseDecimalSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of booleans encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects.
		 */
		AP_bool* (AP_PLUGIN_CALL* getBooleanSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * booleanSequenceElements() and ensures that any changes made to the
		 * sequence elements are reflected in the underlying EPL
		 * objects.  Arrays previously returned by booleanSequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 */
		void (AP_PLUGIN_CALL* releaseBooleanSequenceElements)(const AP_PluginType* obj);

		/** @deprecated
		 * Generate an array of string pointers encapsulating elements
		 * [start..start+length-1] of the sequence object.  This may or may
		 * not require copying the original EPL objects. Strings
		 * pointed to by this sequence are only valid for the lifetime of a
		 * call into the plugin.
		 */
		const AP_char8** (AP_PLUGIN_CALL* getStringSequenceElements)(const AP_PluginType* obj, AP_uint32 start, AP_uint32 length);
		/** @deprecated
		 * Free the resources allocated by *all* preceding calls to
		 * stringSequenceElements() and copies any changes made to the
		 * sequence elements to the underlying EPL objects.
		 * Arrays previously returned by stringSequenceElements()
		 * immediately become invalid.  This function is automatically called
		 * when a sequence AP_Type is destroyed, but can be invoked explicitly
		 * by a plugin to copy changes back during a long-running computation.
		 *
		 * Note that string sequence element arrays do not own the
		 * strings they reference and will never delete a string the
		 * plugin has placed in the array - if the plugin has set a
		 * value, it must delete that string (and it cannot use
		 * the value in the array returned from stringSequenceElement
		 * - plugins will need to keep another pointer to any new
		 * strings they create.
		 */
		void (AP_PLUGIN_CALL* releaseStringSequenceElements)(const AP_PluginType* obj);
	};

	/** [DEPRECATED]\n
	 * AP_Type
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginType_API
	{
		void* reserved;
		/** @deprecated 
		Pointer to function API */
		const struct AP_PluginType_Functions* functions;
	};

	struct AP_PluginType_Stubs {
		struct AP_PluginType_API api;

#ifdef __cplusplus
		AP_TypeDiscriminator getDiscriminator() const {
			return api.functions->getDiscriminator(this);
		}

		AP_PluginChunk* getChunkValue() const {
			return api.functions->getChunkValue(this);
		}

		void setChunkValue(AP_PluginChunk* val) const {
			api.functions->setChunkValue(this, val);
		}

		AP_int64 getIntegerValue() const {
			return api.functions->getIntegerValue(this);
		}

		void setIntegerValue(AP_int64 val) {
			api.functions->setIntegerValue(this, val);
		}

		AP_float64 getFloatValue() const {
			return api.functions->getFloatValue(this);
		}

		void setFloatValue(AP_float64 val) {
			api.functions->setFloatValue(this, val);
		}

		AP_decimal getDecimalValue() const {
			return api.functions->getDecimalValue(this);
		}

		void setDecimalValue(AP_decimal val) {
			api.functions->setDecimalValue(this, val);
		}

		AP_bool getBooleanValue() const {
			return api.functions->getBooleanValue(this);
		}

		void setBooleanValue(AP_bool val) {
			api.functions->setBooleanValue(this, val);
		}

		const AP_char8* getStringValue() const {
			return api.functions->getStringValue(this);
		}

		void setStringValue(const AP_char8* val) {
			api.functions->setStringValue(this, val);
		}

		AP_uint32 getSequenceLength() const {
			return api.functions->getSequenceLength(this);
		}

		AP_TypeDiscriminator getSequenceType() const {
			return api.functions->getSequenceType(this);
		}

		AP_PluginType* getSequenceElement(AP_uint32 index) const {
			return api.functions->getSequenceElement(this, index);
		}

		void copyFrom(const AP_PluginType *other) {
			return api.functions->copyFrom(this, other);
		}

		AP_PluginType* const *getSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getSequenceElements(this, start, length);
		}

		void releaseSequenceElements() const {
			api.functions->releaseSequenceElements(this);
		}

		AP_PluginChunk* const * getChunkSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getChunkSequenceElements(this, start, length);
		}

		void releaseChunkSequenceElements() const {
			api.functions->releaseChunkSequenceElements(this);
		}

		AP_int64* getIntegerSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getIntegerSequenceElements(this, start, length);
		}

		void releaseIntegerSequenceElements() const {
			api.functions->releaseIntegerSequenceElements(this);
		}

		AP_float64* getFloatSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getFloatSequenceElements(this, start, length);
		}

		void releaseFloatSequenceElements() const {
			api.functions->releaseFloatSequenceElements(this);
		}

		AP_decimal* getDecimalSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getDecimalSequenceElements(this, start, length);
		}

		void releaseDecimalSequenceElements() const {
			api.functions->releaseDecimalSequenceElements(this);
		}

		AP_bool* getBooleanSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getBooleanSequenceElements(this, start, length);
		}

		void releaseBooleanSequenceElements() const {
			api.functions->releaseBooleanSequenceElements(this);
		}

		const AP_char8** getStringSequenceElements(AP_uint32 start = 0, AP_uint32 length = ~0U) const {
			return api.functions->getStringSequenceElements(this, start, length);
		}

		void releaseStringSequenceElements() const {
			api.functions->releaseStringSequenceElements(this);
		}
#endif /* __cplusplus */
	};



	/** [DEPRECATED]\n
	 * AP_TypeList function API
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginTypeList_Functions 
	{
		/** @deprecated 
		Return the number of objects in the list */
		AP_uint32 (AP_PLUGIN_CALL* getSize)(const AP_PluginTypeList* list);
		/** @deprecated 
		Return true iff size() == 0 */
		AP_bool (AP_PLUGIN_CALL* isEmpty)(const AP_PluginTypeList* list);
		/** @deprecated 
		Return a pointer to an element of the list */
		const AP_PluginType* (AP_PLUGIN_CALL* getElement)(const AP_PluginTypeList* list, AP_uint32 index);
	};

	/** [DEPRECATED]\n
	 * AP_TypeList
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	struct AP_PluginTypeList_API 
	{
		void* reserved;
		/** @deprecated 
		Pointer to function API */
		const struct AP_PluginTypeList_Functions* functions;
	};

	struct AP_PluginTypeList_Stubs {
		struct AP_PluginTypeList_API api;

#ifdef __cplusplus
		AP_uint32 getSize() const {
			return api.functions->getSize(this);
		}

		AP_bool isEmpty() const {
			return api.functions->isEmpty(this);
		}

		const AP_PluginType* getElement(AP_uint32 index) const {
			return api.functions->getElement(this, index);
		}
#endif /* __cplusplus */
	};


	/** @deprecated
	 * Type of a plugin function.  All functions exported by a plugin library
	 * must follow this interface.
	 *
	 * @param ctx Execution context for this invocation of the function.
	 * @param args Function parameters, passed by reference.
	 * @param rval Function return value, to be filled in by plugin.
	 */
	typedef void (AP_PLUGIN_CALL* AP_PluginFunctionPtr)(const AP_PluginContext* ctx, const AP_PluginTypeList* args, AP_PluginType* rval, AP_TypeDiscriminator rtype);

	/** [DEPRECATED]\n
	 * Plugin function descriptor.  Note that the argument and return types in
	 * this structure are strings (not AP_TypeDiscriminator objects) that use the
	 * same syntax as EPL declarations.  For example, the declare a
	 * function argument as a sequence of integers, the corresponding element of
	 * the paramTypes array would contain "sequence<integer>".
	 * 
	 * @deprecated This class has been deprecated.
	 * 
	 */
	typedef struct
	{
		/** @deprecated 
		Function name */
		const AP_char8* name;

		/** @deprecated 
		Pointer to function implementation */
		AP_PluginFunctionPtr fptr;

		/** @deprecated 
		Argument count */
		AP_uint32 nParams;

		/** @deprecated 
		Argument types - nParams elements, unterminated */
		const AP_char8** paramTypes;

		/** @deprecated 
		Return type */
		const AP_char8* returnType;

	} AP_PluginFunction;
	
	/** @deprecated
	 * Type of a plugin library initialisation function.  Each plugin must
	 * export a single function with this signature, named using the
	 * AP_INIT_FUNCTION_NAME macro, with "C" linkage.  The initialisation function
	 * will be called immediately after the library is loaded by the Engine.
	 *
	 * @param ctx Execution context for the function call.
	 *
	 * @param version Active plugin API version.  The plugin should verify it is
	 * compatible with this version (and return AP_VERSION_MISMATCH_ERROR if not)
	 * and update version to indicate the API version the plugin was compiled
	 * against.
	 *
	 * @param nFunctions The plugin should set this to the number of functions it
	 * exports.
	 *
	 * @param functions The plugin should set this to the address of an array of
	 * AP_Functions structures, of length nFunctions, describing the functions
	 * exported by the plugin.  This data will be read and copied by the plugin
	 * API.
	 *
	 * @return An AP_ErrorCode indicating the success or otherwise of the library
	 * initialisation.
	 */
	typedef AP_PLUGIN_DLL_SYM AP_ErrorCode (AP_PLUGIN_CALL* AP_PluginInitFunctionPtr)(const AP_PluginContext* ctx, AP_uint32* version, AP_uint32* nFunctions, AP_PluginFunction** functions);

	/** @deprecated
	 * Pointer to a plugin library shutdown function.  Each plugin must export a
	 * single function with this signature, named using the
	 * AP_SHUTDOWN_FUNCTION_NAME macro, with "C" linkage.  The shutdown function
	 * will be called immediately before the library is unloaded by the Engine,
	 * either when the last import of the plugin is deleted or the correlator is
	 * shut down.
	 */
	typedef AP_PLUGIN_DLL_SYM AP_ErrorCode (AP_PLUGIN_CALL* AP_PluginShutdownFunctionPtr)(const AP_PluginContext* ctx);

	/** @deprecated
	 * Type of a plugin library version function.  Each plugin must export a
	 * single function with this signature, named using the
	 * AP_LIBRARY_VERSION_FUNCTION_NAME macro, with "C" linkage.  The version
	 * function is called *before* the library initialisation function to
	 * determine the version of the plugin API supported by the library without
	 * triggering any side-effects that the initialisation function might have.
	 * If the function is not present, the plugin API will assume that the library
	 * conforms to an earlier version of the API and proceed accordingly.
	 *
	 * @param ctx Execution context for the function call.
	 *
	 * @param version The plugin should fill this in with the version of the API
	 * it was compiled against.
	 *
	 * @return An AP_ErrorCode indicating the success or otherwise of the function
	 * call.
	 */
	typedef AP_PLUGIN_DLL_SYM AP_ErrorCode (AP_PLUGIN_CALL* AP_PluginVersionFunctionPtr)(const AP_PluginContext* ctx, AP_uint32* version);

	/** @deprecated
	 * Type of a plugin capability function.  Each plugin must export a
	 * single function with this signature, named using the
	 * AP_PLUGIN_GET_CAPABILITIES_FUNCTION_NAME macro, with "C" linkage.  The get capabilities
	 * function is called after the version function and *before* the library
	 * initialisation function to determine the capabilities/ features of the
	 * plugin API supported by the library without
	 * triggering any side-effects that the initialisation function might have.
	 * If the function is not present, the plugin API will assume that the library
	 * conforms to an earlier version of the API and proceed accordingly.
	 *
	 * @param ctx Execution context for the function call.
	 *
	 * @return An AP_Capabilities indicating the capabilities/ properties of the
	 * plugin (multiple values are bitwise-OR'd together)
	 */
	typedef AP_PLUGIN_DLL_SYM AP_Capabilities (AP_PLUGIN_CALL* AP_PluginGetCapabilitiesFunctionPtr)(const AP_PluginContext* ctx);

	/** @deprecated
	 * Type of a plugin library thread ended function.
	 * The function is called by the correlator when a thread ends.
	 * The function is optional, but not implementing it may lead to leaks
	 * as the plugin will not be told if a thread has ended.
	 *
	 * This function pointer is not guaranteed to be called before AP_ShutdownFunctionPtr
	 * Any plugin using this will have to free resources in AP_ShutdownFunctionPtr as if
	 * AP_ThreadEndedFunctionPtr may have not been called.
	 *
	 * @param ctx Execution context for the function call.
	 *
	 * @return An AP_ErrorCode indicating the success or otherwise of the function
	 * call.
	 */
	typedef AP_PLUGIN_DLL_SYM AP_ErrorCode (AP_PLUGIN_CALL* AP_PluginThreadEndedFunctionPtr) (const AP_PluginContext* ctx);

#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* CORRELATOR_PLUGIN_H */
